package com.loafle.overflow.module.generator.service;

import com.loafle.overflow.commons.utils.StringConvertor;
import com.loafle.overflow.crawler.config.Connection;
import com.loafle.overflow.crawler.config.Crawler;
import com.loafle.overflow.crawler.config.Keys;
import com.loafle.overflow.module.infra.model.InfraHost;
import com.loafle.overflow.module.meta.model.MetaCrawler;
import com.loafle.overflow.module.meta.model.MetaSensorItemKey;
import com.loafle.overflow.module.meta.service.MetaSensorItemKeyService;
import com.loafle.overflow.module.meta.type.MetaCrawlerEnum;
import com.loafle.overflow.module.sensor.model.Sensor;
import com.loafle.overflow.module.sensor.model.SensorItem;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by snoop on 17. 9. 6.
 */
@Service("GenerateUtil")
public class GenerateUtil {

    @Autowired
    private MetaSensorItemKeyService metaSensorItemKeyService;

    private Map<Short, Map<Integer, MetaSensorItemKey>> mappingMap = null;

    public Map<Integer, MetaSensorItemKey> initMappingMap(MetaCrawler metaCrawler) {

        if(this.mappingMap == null) {
            this.mappingMap = new HashMap<>();
        }

        Map<Integer, MetaSensorItemKey> resultMap = this.mappingMap.get(metaCrawler.getId());

        if(resultMap != null) {
            return resultMap;
        }

        resultMap = this.metaSensorItemKeyService.readAllMapByCrawler(metaCrawler);
        this.mappingMap.put(metaCrawler.getId(), resultMap);

        return resultMap;
    }

    public Map<String, Object> getAuth(MetaCrawler metaCrawler, com.loafle.overflow.module.infra.model.InfraService infraService) {

        Map<String, Object> auth = new HashMap<>();

       if(metaCrawler.getId() == MetaCrawlerEnum.MYSQL_CRAWLER.getValue()) {
           auth.put("url", "jdbc:mysql://"+ StringConvertor.intToIp(infraService.getHost().getIp())+":"+String.valueOf(infraService.getPort()));
           auth.put("id", "docker"); // FIXME: Auth Info
           auth.put("pw", "qwer5795QWER"); // FIXME: Auth Info
       }

        return auth;
    }

    public Crawler getCrawler(MetaCrawler metaCrawler) {

        Crawler crawler = new Crawler();
        crawler.setName(metaCrawler.getName());

        String container = "";

        if(metaCrawler.getId() == MetaCrawlerEnum.CASSANDRA_CRAWLER.getValue()
                || metaCrawler.getId() == MetaCrawlerEnum.CASSANDRA_CRAWLER.getValue()
                || metaCrawler.getId() == MetaCrawlerEnum.MONGODB_CRAWLER.getValue()
                || metaCrawler.getId() == MetaCrawlerEnum.MSSQL_CRAWLER.getValue()
                || metaCrawler.getId() == MetaCrawlerEnum.ORACLE_CRAWLER.getValue()
                || metaCrawler.getId() == MetaCrawlerEnum.POSTGRESQL_CRAWLER.getValue()) {
            container = "java_proxy";
        }
        else {
            container = "network_proxy";
        }

        crawler.setContainer(container);

        return crawler;
    }

    public com.loafle.overflow.crawler.config.Target createTarget(InfraHost infraHost, Sensor dbSensor) {

        com.loafle.overflow.crawler.config.Target target = new com.loafle.overflow.crawler.config.Target();
        Connection connection = new Connection();
        connection.setIp(StringConvertor.intToIp(infraHost.getIp()));

        if(dbSensor.getCrawler().getId() == MetaCrawlerEnum.WMI_CRAWLER.getValue()) {
            connection.setPort("135");
            connection.setPortType("tcp");
            connection.setSsl(false);

            target.setConnection(connection);

            // FIXME: load crawler Auth Info
            Map<String, Object> auth = new HashMap<>();
            auth.put("id", "administrator");
            auth.put("pw", "dbseogns18");

            target.setAuth(auth);

        }

        return target;
    }


    public Map<String, List<MetaSensorItemKey>> sortItems(List<SensorItem> sensorItems, Map<Integer, MetaSensorItemKey> keyMap) {

        Map<String, List<MetaSensorItemKey>> metricMap = new HashMap<>();

        MetaSensorItemKey itemKey = null;

        for(SensorItem sItem : sensorItems) {
            itemKey = keyMap.get(sItem.getItem().getId());
            if(metricMap.containsKey(itemKey.getFroms()) == false) {
                metricMap.put(itemKey.getFroms(), new ArrayList<>());
            }
            metricMap.get(itemKey.getFroms()).add(itemKey);
        }

        return metricMap;
    }

    public Keys createKeys(MetaSensorItemKey itemKey) {
        Keys keys = new Keys();
        keys.setKey(itemKey.getKey());
        keys.setMetric(itemKey.getItem().getKey());
        return keys;
    }
}
